<?php

namespace Drp\NativeEventApp\Auth;

use Error;
use League\OAuth2\Server\Exception\OAuthServerException;
use TypeError;

class CodeGenerator
{
    /** @var string */
    private $characterSubset = '';

    public function __construct(string $characterSubset)
    {
        $this->characterSubset = $characterSubset;
    }

    public function authRequestCode(int $length = 5): string
    {
        return substr(
            str_shuffle(
                str_repeat($this->characterSubset, max($length, strlen($this->characterSubset)))
            ),
            0,
            $length
        );
    }

    public function authToken(int $length = 40): string
    {
        try {
            return bin2hex(random_bytes($length));
        } catch (TypeError $e) {
            throw OAuthServerException::serverError('An unexpected error has occurred', $e);
        } catch (Error $e) {
            throw OAuthServerException::serverError('An unexpected error has occurred', $e);
        } catch (\Throwable $e) {
            // If you get this message, the CSPRNG failed hard.
            throw OAuthServerException::serverError('Could not generate a random string', $e);
        }
    }
}
