<?php

namespace Drp\NativeEventApp\Auth;

use DateInterval;
use Drp\NativeEventApp\Contracts\MagicLinkSigner;
use Drp\NativeEventApp\Contracts\QrCode;
use Drp\NativeEventApp\Contracts\TokenAuthenticatable;
use Illuminate\Support\InteractsWithTime;

class MagicLinkGenerator implements \Drp\NativeEventApp\Contracts\MagicLinkGenerator
{
    use InteractsWithTime;

    /**  @var null|string */
    protected $url = null;

    /**  @var MagicLinkSigner */
    private $linkSigner;

    /** @var QrCode */
    private $qrCode;

    /** @var string */
    private $graphQlUrl;

    public function __construct(
        MagicLinkSigner $linkSigner,
        QrCode $qrCode,
        string $graphQlUrl,
        DateInterval $magicLinkTTL
    ) {
        $this->linkSigner = $linkSigner;
        $this->qrCode = $qrCode;
        $this->graphQlUrl = $graphQlUrl;
        $this->magicLinkTTL = $magicLinkTTL;
    }

    public function qrCode(TokenAuthenticatable $authenticatable, int $width = 500): string
    {
        $url = $this->url($authenticatable);

        return $this->qrCode->generate($width, $url);
    }

    public function url(TokenAuthenticatable $authenticatable): string
    {
        if ($this->url) {
            return $this->url;
        }

        $authId = $authenticatable->getId();
        $expiry = $this->availableAt($this->magicLinkTTL);

        return $this->url = 'https://nea.drpg.dev/in?' . $this->buildQuery($authId, $expiry);
    }

    private function buildQuery(string $authId, int $expiry): string
    {
        return http_build_query(
            [
                'url' => $this->graphQlUrl,
                'id' => $authId,
                'signature' => $this->linkSigner->sign($authId, $expiry),
                'expiry' => $expiry,
            ]
        );
    }
}
