<?php

namespace Drp\NativeEventApp\Test\Api;

use Drp\NativeEventApp\Models\Event;
use Spatie\Snapshots\MatchesSnapshots;

class EventTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function returns_socket_server_url(): void
    {
        $this->registerFakeEvent();

        config(['native-event-app.socket_server.host' => 'http://localhost:8000']);

        $query = '{
            events(id:1){
                id
                socketServerUrl
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function it_can_get_the_events_for_the_authenticated(): void
    {
        $this->registerFakeEvent();
        factory(Event::class)->create();

        $query = '{
            events {
                id
                title
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function cant_retrieve_an_event_the_authenticated_user_does_not_have_access_to(): void
    {
        $this->registerFakeEvent();
        $eventB = factory(Event::class)->create();

        $query = "{
            events(id: $eventB->id) {
                id
                title
            }
        }";

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function event_can_have_styles()
    {
        $event = factory(Event::class)->create();
        $event->addStyles([
            'Container' => [
                'container' => [
                    'padding' => 16,
                ],
            ],
        ]);

        $query = '{
            events(id:1) {
                id
                styles
            }
        }';

        $response = $this->authenticatedGraphQlRequest($query, $event);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function event_can_have_a_logo()
    {
        $event = factory(Event::class)->create([
            'logo' => 'https://www.example.com/image.png',
        ]);

        $query = '{
            events(id:1) {
                id
                logo
            }
        }';

        $response = $this->authenticatedGraphQlRequest($query, $event);

        $this->assertMatchesJsonSnapshot($response);
    }
}
