<?php


namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Auth\AuthInfoFinder;
use Drp\NativeEventApp\Test\Support\Fakes\MagicLinkSigner;
use Drp\NativeEventApp\Test\Support\Fakes\Delegate;
use Drp\NativeEventApp\Test\TestCase;


class AuthInfoFinderTest extends TestCase
{
    public function setUp(): void
    {
        $reflection = new \ReflectionClass(AuthInfoFinder::class);
        tap($reflection->getProperty('authResolver'))->setAccessible(true)->setValue(null);
        tap($reflection->getProperty('currentUserResolver'))->setAccessible(true)->setValue(null);
    }

    /**
     * @test
     */
    public function throws_exception_if_no_auth_resolver_is_configured()
    {
        $this->expectException(\RuntimeException::class);
        $authInfoFinder = new AuthInfoFinder();
        $authInfoFinder->retrieve(1);
    }

    /**
     * @test
     */
    public function throws_exception_if_no_current_user_resolver_is_configured()
    {
        $this->expectException(\RuntimeException::class);
        $authInfoFinder = new AuthInfoFinder();
        $authInfoFinder->current();
    }

    /**
     * @test
     */
    public function returns_current_user_using_resolver()
    {
        $delegate = new Delegate;
        $fakeSigner = new MagicLinkSigner;
        $authInfoFinder = new AuthInfoFinder($fakeSigner);
        $authInfoFinder->setCurrentUserResolver(function () use ($delegate) {
            return $delegate;
        });

        $this->assertEquals($delegate, $authInfoFinder->current());
    }
}
