<?php

namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Quiz\Client\QuizClient;
use Drp\NativeEventApp\Test\TestCase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Middleware;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;

class QuizClientTest extends TestCase
{
    /** @test */
    public function can_make_post_request()
    {
        $container = [];
        $client = $this->mockGuzzleClient($container);
        $quizClient = new QuizClient($client);

        $quizClient->post('1/end');

        $this->assertCount(1, $container);
        tap($container[0]['request'], function (Request $request) {
            $this->assertEquals('POST', $request->getMethod());
            $this->assertEquals('/api/quiz/1/end', $request->getUri());
            $this->assertEquals('[]', $request->getBody()->getContents());
        });
    }

    /** @test */
    public function can_make_post_request_with_body()
    {
        $container = [];
        $client = $this->mockGuzzleClient($container);
        $quizClient = new QuizClient($client);

        $quizClient->post('', ['test' => 1]);

        $this->assertCount(1, $container);
        tap($container[0]['request'], function (Request $request) {
            $this->assertEquals('POST', $request->getMethod());
            $this->assertEquals('/api/quiz', $request->getUri()->getPath());
            $this->assertEquals('{"test":1}', $request->getBody()->getContents());
        });
    }

    /** @test */
    public function can_make_delete_request()
    {
        $container = [];
        $client = $this->mockGuzzleClient($container);
        $quizClient = new QuizClient($client);

        $quizClient->delete('1');

        $this->assertCount(1, $container);
        tap($container[0]['request'], function (Request $request) {
            $this->assertEquals('DELETE', $request->getMethod());
            $this->assertEquals('/api/quiz/1', $request->getUri());
            $this->assertEquals('', $request->getBody()->getContents());
        });
    }

    private function mockGuzzleClient(array &$container): Client
    {
        $history = Middleware::history($container);

        $mock = new MockHandler([
            new Response(200),
        ]);

        $stack = HandlerStack::create($mock);
        $stack->push($history);

        return new Client(['handler' => $stack]);
    }
}
